/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.minesweeper;

import org.eclipse.ui.plugin.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.core.resources.*;
import es.org.chemi.games.util.Constants;
import java.util.*;
import java.util.Hashtable;

/**
 * The main plugin class to be used in the desktop.
 */
public class MinesweeperPlugin extends AbstractUIPlugin 
{
	//The shared instance.
	private static MinesweeperPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	/**
	 * The constructor.
	 */
	public MinesweeperPlugin(IPluginDescriptor descriptor) 
	{
		super(descriptor);
		plugin = this;
		
		// Tracing.
		if(MinesweeperPlugin.getDefault().isDebugging())
			System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Loading " + this.getDescriptor().getUniqueIdentifier() + " plug-in (version: " + this.getDescriptor().getVersionIdentifier() + ")...");
			
		try 
		{
			resourceBundle= ResourceBundle.getBundle("edu.chemi.games.minesweeper.MinesweeperPluginResources");
		}
		catch (MissingResourceException x) 
		{
			resourceBundle = null;
		}
		
		// Store default values.
		IPreferenceStore store = this.getDefault().getPreferenceStore();
		
		// Store default mode.
		store.setDefault("type",Constants.MODE_EXPERT);

		// Store default records.
		store.setDefault(Constants.RECORD_NAME_MODE_EXPERT,"Anonymous");
		store.setDefault(Constants.RECORD_TIME_MODE_EXPERT,999);		
		store.setDefault(Constants.RECORD_NAME_MODE_INTERMEDIATE,"Anonymous");
		store.setDefault(Constants.RECORD_TIME_MODE_INTERMEDIATE,999);		
		store.setDefault(Constants.RECORD_NAME_MODE_BEGINNER,"Anonymous");
		store.setDefault(Constants.RECORD_TIME_MODE_BEGINNER,999);	
	}

	/**
	 * Returns the shared instance.
	 */
	public static MinesweeperPlugin getDefault() 
	{
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() 
	{
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) 
	{
		ResourceBundle bundle= MinesweeperPlugin.getDefault().getResourceBundle();
		try 
		{
			return bundle.getString(key);
		}
		catch (MissingResourceException e) 
		{
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() 
	{
		return resourceBundle;
	}
}