package es.org.chemi.games.minesweeper.events;

import es.org.chemi.games.minesweeper.MinesweeperPlugin;
import es.org.chemi.games.minesweeper.core.CellInfo;
import es.org.chemi.games.minesweeper.ui.Cell;
import es.org.chemi.games.minesweeper.ui.MainView;
import es.org.chemi.games.util.Constants;
import es.org.chemi.games.util.ResourceManager;
import java.util.Calendar;
import java.util.Vector;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Control;

public class CellListener implements MouseListener 
{
	// Variables
	private String traceEvents = Platform.getDebugOption("es.org.chemi.games.minesweeper/debug/events");
	private MainView view = null;
	private Vector list = null;					
	private boolean grayed = false; // Switch used to check if buttons have been grayed.	

	public CellListener(MainView view)
	{
		this.view = view;
	} 	

	/**
	 * @see MouseListener#mouseDoubleClick(MouseEvent)
	 */
	public void mouseDoubleClick(MouseEvent e) 
	{
	}

	/**
	 * @see MouseListener#mouseDown(MouseEvent)
	 */
	public void mouseDown(MouseEvent e) 
	{
		if(((Cell)e.getSource()).isButton())
			mouseDownForButtonType(e);
		else
			mouseDownForLabelType(e);		
	}

	/**
	 * @see MouseListener#mouseUp(MouseEvent)
	 */
	public void mouseUp(MouseEvent e) 
	{
		if(((Cell)e.getSource()).isButton())
			mouseUpForButtonType(e);
		else
			mouseUpForLabelType(e);				
	}
	
	public void mouseDownForButtonType(MouseEvent e)
	{
		// Variables.
		Cell cell = null;
		CellInfo cellInfo = null;
		
		// Tracing.
		if(traceEvents != null && traceEvents.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_EVENTS + "] - " + Calendar.getInstance().getTime() + " - Calling method mouseDown() over a button, button=" + e.button + ", stateMask=" + e.stateMask);		
		
		// Check which mouse button has been pressed.
		if(e.button == 1)
		{
			// Check if timer is running.
			if(!view.getTimer().isRunning())
			{
				view.getTimer().start();
				Thread thread = new Thread(view.getTimer());
				thread.start();
			}	
			
			// Show surprise icon.
			cell = (Cell)e.getSource();
			cellInfo = (CellInfo)cell.getData();
			if(!cellInfo.isFlag())
				view.getMainButton().setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SURPRISE));
		}
		// Right button has been pressed.
		else
		{
			cell = (Cell)e.getSource();
			cellInfo = (CellInfo)cell.getData();
			
			if(!cellInfo.isFlag() && !cellInfo.isQuestion() && !view.getCounter().isZero())
			{
				cellInfo.setFlag(true);
				cell.setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_FLAG));
				view.getCounter().decrease();
			}
			else if(cellInfo.isFlag())
			{
				cellInfo.setFlag(false);
				cellInfo.setQuestion(true);
				cell.setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_QUESTION));	
				view.getCounter().increase();	
			}
			else
			{
				cellInfo.setQuestion(false);
				cell.setImage(null);					
			}
			cell.redraw();									
		}	
	}
	
	public void mouseUpForButtonType(MouseEvent e)
	{
		// Variables.
		Control[] cells	= null;
		CellInfo cellInfo = null;
		Cell cell = null;
		
		// Tracing.
		if( traceEvents != null && traceEvents.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_EVENTS + "] - " + Calendar.getInstance().getTime() + " - Calling method mouseUp(), button=" + e.button + ", stateMask=" + e.stateMask);		

		// Check which mouse button has been pressed.		
		if(e.button == 1)
		{
			// Get cell information.
			cell = (Cell)e.getSource();
			cellInfo = (CellInfo)cell.getData();						

			// Check if it has a flag.
			if(!cellInfo.isFlag())
			{
				// Show field or mine.
				cell.setType(Constants.TYPE_LABEL);

				if(cellInfo.isMine())
				{
					// Stop timer.
					view.getTimer().stop();										
					
					// Create mine label.
					cell.setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_EXPLOSION));
					cell.setBackground(ResourceManager.getResourceManager().getColor(Constants.COLOR_RED));
					cell.redraw();
					
					// Show sad icon.
					view.getMainButton().setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SAD));
					
					// Show all mines.
					view.getController().showAllMines();				
				}
				else
				{
					// Create warning label.
					if(cellInfo.getWarning() == 0)
					{
						cell.redraw();
						
						// Create the list.
						list = new Vector();

						// Tracing.
						if(MinesweeperPlugin.getDefault().isDebugging())
							System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Cleaning zero mines zone...");		

						// Clean all cells around this one
						view.getController().showAllZeroWarningsAround(cellInfo,list);
						
						cells = view.getMinesField().getChildren();
						
						for(int i=1; i<list.size(); i++) // i=1 because the first cell has been cleaned already.
						{
							cellInfo = (CellInfo)list.elementAt(i);
							if(!cellInfo.isFlag())
							{
								cell = (Cell)cells[cellInfo.getLocation().y + cellInfo.getLocation().x * view.getController().getPreferences().getColumns()];
								cell.setType(Constants.TYPE_LABEL);
								
								if(cellInfo.getWarning() != 0)
									cell.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(cellInfo.getWarning())+"mines.gif"));
								else if(cellInfo.isQuestion())
									cell.setImage(null);
									
								cell.redraw();
							}
						}
						
						// Free the list.
						list = null;
					}
					else
					{
						cell.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(cellInfo.getWarning())+"mines.gif"));
						cell.redraw();
					}

					// Show smile icon.
					view.getMainButton().setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SMILE));
				}
			}					
		}
			
		// Check the end of the game.
		if(view.getCounter().isZero())
			view.getController().finishGame();		
	}

	
	public void mouseDownForLabelType(MouseEvent e)
	{
		// Variables.
		CellInfo cellInfo = null;
		Control[] cells = null;		
		Cell cell = null;	
		CellInfo aux = null;	
		
		// Tracing.
		if(traceEvents != null && traceEvents.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_EVENTS + "] - " + Calendar.getInstance().getTime() + " - Calling method mouseDown() over a CellView, button=" + e.button + ", stateMask=" + e.stateMask);		

		// Show surprise icon.
		view.getMainButton().setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SURPRISE));
		
		// Check that both buttons (right and left) have been pressed at the same time.
		if(e.button == 3 && e.stateMask == SWT.BUTTON1)
		{
			// Create the list.
			list = new Vector();

			// Tracing.
			if(MinesweeperPlugin.getDefault().isDebugging())
				System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Showing cells around...");		

			list = view.getController().showAllCellsAround((CellInfo)((Cell)e.getSource()).getData());			
			
			cellInfo = (CellInfo)((Cell)e.getSource()).getData();
			int warnings = cellInfo.getWarning();
			int check = 0;
			
			// Check that all mines around have been checked.
			for(int i=0; i<list.size(); i++)
			{
				cellInfo = (CellInfo)list.elementAt(i);
				if(cellInfo.isFlag())
					check++;
			}
			
			if(warnings <= check)
			{
				cells = view.getMinesField().getChildren();
											
				for(int i=0; i<list.size(); i++)
				{
					cellInfo = (CellInfo)list.elementAt(i);
					cell = (Cell)cells[cellInfo.getLocation().y + cellInfo.getLocation().x * view.getController().getPreferences().getColumns()];					
					if(cell.isButton() && !cellInfo.isFlag())
					{
						cell.setType(Constants.TYPE_LABEL);
								
						if(cellInfo.isMine())
						{
							// Stop timer.
							view.getTimer().stop();										
						
							// Create mine label.
							cell.setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_EXPLOSION));
							cell.setBackground(ResourceManager.getResourceManager().getColor(Constants.COLOR_RED));
						
							// Show sad icon.
							view.getMainButton().setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SAD));
						}
						else if(cellInfo.getWarning() != 0)
							cell.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(cellInfo.getWarning())+"mines.gif"));							
						else
							aux = cellInfo;
							
						cell.redraw();
						
						// Show all mines.
						if(cellInfo.isMine())
							view.getController().showAllMines();
					}
				}
				
				// Free the list.
				list = null;
				
				// Check if any zero warning cell has been shown.
				if(aux != null)
				{
					// Create a new the list.
					list = new Vector();

					// Clean all cells around this one
					view.getController().showAllZeroWarningsAround(aux, list);
					
					cells = view.getMinesField().getChildren();
					
					for(int i=1; i<list.size(); i++) // i=1 because the first cell has been cleaned already.
					{
						cellInfo = (CellInfo)list.elementAt(i);
						if(!cellInfo.isFlag())
						{
							cell = (Cell)cells[cellInfo.getLocation().y + cellInfo.getLocation().x * view.getController().getPreferences().getColumns()];
							cell.setType(Constants.TYPE_LABEL);
							
							if(cellInfo.getWarning() != 0)
								cell.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(cellInfo.getWarning())+"mines.gif"));
								
							cell.redraw();
						}
					}
					
					// Free the list.
					list = null;
				}
			}
			// It is not possible to show cells around yet.
			else
			{
				grayed = true;
				
				cells = view.getMinesField().getChildren();
											
				for(int i=0; i<list.size(); i++)
				{
					cellInfo = (CellInfo)list.elementAt(i);
					cell = (Cell)cells[cellInfo.getLocation().y + cellInfo.getLocation().x * view.getController().getPreferences().getColumns()];					
					if(cell.isButton() && !cellInfo.isFlag())
					{
						cell.setImage(null);
						cell.setType(Constants.TYPE_SHADOW);
						cell.redraw();
					}
				}
			}				
		}
	}
	
	public void mouseUpForLabelType(MouseEvent e)
	{
		// Variables.
		Control[] cells = null;	
		CellInfo cellInfo = null;
		Cell cell = null;			
		
		// Show smile icon.
		view.getMainButton().setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SMILE));
			
		// Check if buttons have been grayed.
		if(grayed)
		{
			grayed = false;
			cells = view.getMinesField().getChildren();
												
			for(int i=0; i<list.size(); i++)
			{
				cellInfo = (CellInfo)list.elementAt(i);
				cell = (Cell)cells[cellInfo.getLocation().y + cellInfo.getLocation().x * view.getController().getPreferences().getColumns()];					
				if(cell.isShadow())
				{
					if(cellInfo.isQuestion())
						cell.setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_QUESTION));
					cell.setType(Constants.TYPE_BUTTON);
					cell.redraw();
				}
			}
			// Free list.
			list = null;
		}
		
		// Check the end of the game.
		if(view.getCounter().isZero())
			view.getController().finishGame();
	}
}