package es.org.chemi.games.minesweeper.ui;

import es.org.chemi.games.minesweeper.core.CellInfo;
import es.org.chemi.games.util.Constants;
import es.org.chemi.games.util.ResourceManager;
import java.util.Calendar;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

public class Cell extends Canvas 
{
	private String traceUI =  Platform.getDebugOption("es.org.chemi.games.minesweeper/debug/ui");
	private Image image = null;
	private int type = Constants.TYPE_BUTTON;
	
	/**
	 * Constructor for Cell
	 */
	public Cell(Composite parent, int style)
	{
		super(parent, SWT.NONE);
		
		// Tracing.
		if(traceUI != null && traceUI.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " - Creating a Cell");		

		if(ResourceManager.getResourceManager().getColor(Constants.COLOR_DARK_GRAY) == null)
			ResourceManager.getResourceManager().putColor(Constants.COLOR_DARK_GRAY,new Color(null,132,130,132));
		if(ResourceManager.getResourceManager().getColor(Constants.COLOR_LIGHT_GRAY) == null)
			ResourceManager.getResourceManager().putColor(Constants.COLOR_LIGHT_GRAY,new Color(null,198,195,198));
		if(ResourceManager.getResourceManager().getColor(Constants.COLOR_WHITE) == null)
			ResourceManager.getResourceManager().putColor(Constants.COLOR_WHITE,new Color(null,255,255,255));
		if(ResourceManager.getResourceManager().getColor(Constants.COLOR_PLATFORM) == null)
			ResourceManager.getResourceManager().putColor(Constants.COLOR_PLATFORM,this.getBackground());
		
		this.addPaintListener(new PaintListener()
		{
			public void paintControl(PaintEvent e)
			{
				Cell.this.paintControl(e);
			}
		});
	}
	
	private void paintControl(PaintEvent e)
	{
		// Tracing.
		if(traceUI != null && traceUI.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " - Redrawing a Cell");		
		
		GC gc = e.gc;
		
		if(this.type == Constants.TYPE_BUTTON)
		{
			gc.setForeground(ResourceManager.getResourceManager().getColor(Constants.COLOR_WHITE));
			gc.drawLine(0,0,getSize().x-2,0);	
			gc.drawLine(0,1,getSize().x-3,1);
			gc.drawLine(0,0,0,getSize().y-2);
			gc.drawLine(1,0,1,getSize().y-3);
			gc.setForeground(ResourceManager.getResourceManager().getColor(Constants.COLOR_DARK_GRAY));
			gc.drawLine(2,getSize().y-2,getSize().x-1,getSize().y-2);
			gc.drawLine(1,getSize().y-1,getSize().x-1,getSize().y-1);
			gc.drawLine(getSize().x-2,2,getSize().x-2,getSize().y-1);
			gc.drawLine(getSize().x-1,1,getSize().x-1,getSize().y-1);
		}
		else
		{
			gc.setForeground(ResourceManager.getResourceManager().getColor(Constants.COLOR_DARK_GRAY));
			gc.drawLine(0,0,getSize().x-1,0);	// Top line.
			gc.drawLine(0,0,0,getSize().y-1);	// Left line.
			if(((CellInfo)getData()).isRightBorder())
				gc.drawLine(getSize().x-1,0,getSize().x-1,getSize().y-1);
			if(((CellInfo)getData()).isDownBorder())
				gc.drawLine(0,getSize().y-1,getSize().x-1,getSize().y-1);
		}
		if(image != null)
			gc.drawImage(image,(this.getSize().x - image.getBounds().width)/2 + 1,(this.getSize().y - image.getBounds().height)/2 + 1);
	}
	
	public void setImage(Image image)
	{
		this.image = image;
	}
	
	public Image getImage()
	{
		return this.image;
	}
	
	public void setType(int type)
	{
		this.type = type;
	}
	
	public boolean isButton()
	{
		if((type & Constants.TYPE_BUTTON) > 0)
			return true;
		else
			return false;
	}
	
	public boolean isLabel()
	{
		if((type & Constants.TYPE_LABEL) > 0)
			return true;
		else
			return false;		
	}
	
	public boolean isShadow()
	{
		if((type & Constants.TYPE_SHADOW) > 0)
			return true;
		else
			return false;		
	}
	
	public Point computeSize(int wHint, int hHint, boolean changed)
	{
		return new Point(this.getSize().x,this.getSize().y);
	}
}