/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.minesweeper.ui;

import es.org.chemi.games.util.Constants;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class CustomDialog extends Dialog implements ModifyListener
{
	MainView view = null;
    Text rowsValue = null;
    Text columnsValue = null;
    Text minesValue = null;
    Button ok = null;

	/**
	 * Constructor for CustomDialog
	 */
	public CustomDialog(Shell shell, MainView view) 
	{
		super(shell);
		this.view = view;
	}
	
	protected void configureShell(Shell newShell) 
	{
 	   super.configureShell(newShell);
    	newShell.setText("Custom Field");
	}
	
	protected void createButtonsForButtonBar(Composite parent) 
	{
		// create OK and Cancel buttons by default
		ok = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		ok.setEnabled(false);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}
	
	protected Control createDialogArea(Composite parent) 
	{
	    Composite composite = (Composite)super.createDialogArea(parent);
	    
	    GridLayout gridLayout = new GridLayout();
	    gridLayout.numColumns = 2;
	    gridLayout.horizontalSpacing = 25;
	    gridLayout.marginWidth = 20;
	    
	    composite.setLayout(gridLayout);
	    
	    Label rows = new Label(composite,SWT.NONE);
	    rows.setText("Rows:");
	    rowsValue = new Text(composite,SWT.BORDER);
	    rowsValue.addModifyListener(this);
	    Label columns = new Label(composite,SWT.NONE);
	    columns.setText("Columns:");
	    columnsValue = new Text(composite,SWT.BORDER);
	    columnsValue.addModifyListener(this);
	    Label mines = new Label(composite,SWT.NONE);	    	    	    	    
	    mines.setText("Mines:");
	    minesValue = new Text(composite,SWT.BORDER);
	    minesValue.addModifyListener(this);
	    
    	return composite;
	}
	
	public void modifyText(ModifyEvent e)
	{
		try
		{
			Integer.parseInt(rowsValue.getText());
			Integer.parseInt(columnsValue.getText());
			Integer.parseInt(minesValue.getText());						
			ok.setEnabled(true);
		}
		catch(NumberFormatException ex)
		{
			ok.setEnabled(false);
		}
	}
	
	protected void okPressed()
	{
		view.updateActionsUI(Constants.MODE_CUSTOM);
		view.getController().refresh(Integer.valueOf(minesValue.getText()).intValue(),Integer.valueOf(rowsValue.getText()).intValue(),Integer.valueOf(columnsValue.getText()).intValue());
		view.getController().changeMode(Constants.MODE_CUSTOM);
		super.okPressed();		
	}
}

