/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.minesweeper.ui;

import es.org.chemi.games.minesweeper.MinesweeperPlugin;
import es.org.chemi.games.minesweeper.actions.BeginnerAction;
import es.org.chemi.games.minesweeper.actions.CustomAction;
import es.org.chemi.games.minesweeper.actions.ExitAction;
import es.org.chemi.games.minesweeper.actions.ExpertAction;
import es.org.chemi.games.minesweeper.actions.IntermediateAction;
import es.org.chemi.games.minesweeper.actions.NewAction;
import es.org.chemi.games.minesweeper.actions.RecordsAction;
import es.org.chemi.games.minesweeper.events.MainButtonListener;
import es.org.chemi.games.minesweeper.core.CellInfo;
import es.org.chemi.games.minesweeper.core.Controller;
import es.org.chemi.games.minesweeper.util.Counter;
import es.org.chemi.games.minesweeper.util.Preferences;
import es.org.chemi.games.minesweeper.util.Timer;
import es.org.chemi.games.util.Constants;
import es.org.chemi.games.util.NewGridLayout;
import es.org.chemi.games.util.ResourceManager;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.Hashtable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.DeviceData;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;


public class MainView extends ViewPart
{
	private Composite minesField = null;
	private Controller controller = null;
	private Button mainButton = null;
	private Counter counter = null;
	private Timer timer = null;
	
	private ExpertAction expertAction = null;
	private IntermediateAction intermediateAction = null;
	private BeginnerAction beginnerAction = null;
	private CustomAction customAction = null;
	
	private URL url = null;
	
	/**
	 * @see WorkbenchPart#setFocus()
	 */
	public void setFocus() 
	{
	}
	
	public void init(IViewSite site, IMemento memento)
	{
		try
		{
			init(site);
		}
		catch(PartInitException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));
		}
		
		// Loading images.
		loadImages();
		
		// Loading colors.
		loadColors();
	}

	private void loadImages() 
	{
		// Loading images
		URL imageLocation = null;
		
		try
		{
			// Tracing.
			if(MinesweeperPlugin.getDefault().isDebugging())
				System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Loading resources...");
		
			url = new URL(MinesweeperPlugin.getDefault().getDescriptor().getInstallURL(),"icons/");
			imageLocation = new URL(url,Constants.IMAGE_SMILE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_SMILE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_WRONG);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_WRONG,new Image(null,imageLocation.openConnection().getInputStream()));			
			imageLocation = new URL(url,Constants.IMAGE_SURPRISE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_SURPRISE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_EXPLOSION);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_EXPLOSION,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_FLAG);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_FLAG,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_QUESTION);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_QUESTION,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_SAD);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_SAD,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_HAPPY);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_HAPPY,new Image(null,imageLocation.openConnection().getInputStream()));			
			imageLocation = new URL(url,Constants.IMAGE_ONE_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_ONE_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_TWO_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_TWO_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_THREE_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_THREE_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_FOUR_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_FOUR_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_FIVE_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_FIVE_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_SIX_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_SIX_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_SEVEN_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_SEVEN_MINE,new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(url,Constants.IMAGE_EIGHT_MINE);
			ResourceManager.getResourceManager().putImage(Constants.IMAGE_EIGHT_MINE,new Image(null,imageLocation.openConnection().getInputStream()));			
			//imageLocation = new URL(aux,Constants.IMAGE_MARK);
			//ResourceManager.getResourceManager().putImage(Constants.IMAGE_MARK,new Image(null,imageLocation.openConnection().getInputStream()));									
			url = new URL(url,Constants.IMAGE_MARK);
		}			
		catch(MalformedURLException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));			
		}
		catch(IOException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));			
		}
	}	

	private void loadColors() 
	{
		// Loading colors.
		ResourceManager.getResourceManager().putColor(Constants.COLOR_RED,new Color(null,255,0,0));
		ResourceManager.getResourceManager().putColor(Constants.COLOR_DARK_GRAY,new Color(null,132,130,132));
		ResourceManager.getResourceManager().putColor(Constants.COLOR_LIGHT_GRAY,new Color(null,198,195,198));
		ResourceManager.getResourceManager().putColor(Constants.COLOR_WHITE,new Color(null,255,255,255));
	}
		
	public void createPartControl(Composite parent) 
	{
		// Tracing.
		if(MinesweeperPlugin.getDefault().isDebugging())
			System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Starting the creation of Minesweeper view...");
		
		// Restore state.
		restoreState(MinesweeperPlugin.getDefault().getPreferenceStore());
		
		// Create UI.
		open(parent);
		
		// Tracing.
		if(MinesweeperPlugin.getDefault().isDebugging())
			System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Finishing the creation of Minesweeper view...");		
	}
		
	/**
	 * 
	 */
	private void open(Composite parent) 
	{
		// Setting Composite Layout.
		GridLayout gridLayout = new GridLayout(); 
		gridLayout.numColumns = 3;
		gridLayout.marginWidth = 10;		
		gridLayout.marginHeight = 10;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 10;
		gridLayout.makeColumnsEqualWidth = true;
		parent.setLayout(gridLayout);
		
		// Creating Panels.
		createView(parent);
		
		// Creating Actions.
		createActions();
	}
	
	private void createView(Composite parent)
	{
		// Create the counter.
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.BEGINNING;
		gridData.verticalAlignment = GridData.CENTER;				
		this.counter = new Counter(parent,SWT.NONE,this.controller.getPreferences().getMinesCount());
		this.counter.setLayoutData(gridData);
		
		// Create the main button.
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.CENTER;
		gridData.verticalAlignment = GridData.CENTER;		
		this.mainButton = new Button(parent,SWT.PUSH);
		this.mainButton.setImage(ResourceManager.getResourceManager().getImage(Constants.IMAGE_SMILE));
		this.mainButton.setLayoutData(gridData);
		this.mainButton.addMouseListener(new MainButtonListener(this));
		
		// Create the timer.
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		gridData.verticalAlignment = GridData.CENTER;		
		this.timer = new Timer(parent,SWT.NONE);
		this.timer.setLayoutData(gridData);

		// Create the mines field.
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;	
		gridData.grabExcessHorizontalSpace = true;	
		gridData.grabExcessVerticalSpace = true;			
		minesField = new Composite(parent,SWT.NONE);
		minesField.setLayoutData(gridData);

		GridLayout layout = new GridLayout();
		layout.numColumns = this.controller.getPreferences().getColumns();
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.horizontalSpacing = 0;
		layout.verticalSpacing = 0;
		layout.makeColumnsEqualWidth = true;
		minesField.setLayout(layout);
		
		// Initialize the mines panel.
		controller.initMinesField(minesField);
	}
	
	private void createActions()
	{
		IMenuManager mgr = this.getViewSite().getActionBars().getMenuManager();
		mgr.add(new NewAction("New",this));
		mgr.add(new Separator());
		mgr.add(beginnerAction = new BeginnerAction("Beginner",this));		
		mgr.add(intermediateAction = new IntermediateAction("Intermediate",this));
		mgr.add(expertAction = new ExpertAction("Expert",this));
		mgr.add(customAction = new CustomAction("Custom...",this));								
		mgr.add(new Separator());
		mgr.add(new RecordsAction("Best Times...",this));		
		mgr.add(new Separator());
		mgr.add(new ExitAction("Exit"));

		if(controller.getPreferences().getMode().equals(Constants.MODE_EXPERT))
			expertAction.setImageDescriptor(ImageDescriptor.createFromURL(url));				
		else if(controller.getPreferences().getMode().equals(Constants.MODE_INTERMEDIATE))
			intermediateAction.setImageDescriptor(ImageDescriptor.createFromURL(url));						
		else if(controller.getPreferences().getMode().equals(Constants.MODE_BEGINNER))
			beginnerAction.setImageDescriptor(ImageDescriptor.createFromURL(url));						
		else
			customAction.setImageDescriptor(ImageDescriptor.createFromURL(url));						
	}
	
	public void updateActionsUI(String mode)
	{
		if(mode.equals(Constants.MODE_EXPERT))
		{
			expertAction.setImageDescriptor(ImageDescriptor.createFromURL(url));							
			intermediateAction.setImageDescriptor(null);										
			beginnerAction.setImageDescriptor(null);										
			customAction.setImageDescriptor(null);										
		}
		else if(mode.equals(Constants.MODE_INTERMEDIATE))
		{
			expertAction.setImageDescriptor(null);							
			intermediateAction.setImageDescriptor(ImageDescriptor.createFromURL(url));										
			beginnerAction.setImageDescriptor(null);										
			customAction.setImageDescriptor(null);										
		}
		else if(mode.equals(Constants.MODE_BEGINNER))
		{
			expertAction.setImageDescriptor(null);							
			intermediateAction.setImageDescriptor(null);										
			beginnerAction.setImageDescriptor(ImageDescriptor.createFromURL(url));										
			customAction.setImageDescriptor(null);										
		}
		else
		{
			expertAction.setImageDescriptor(null);							
			intermediateAction.setImageDescriptor(null);										
			beginnerAction.setImageDescriptor(null);										
			customAction.setImageDescriptor(ImageDescriptor.createFromURL(url));										
		}
	}
	
	// Restore State.
	private void restoreState(IPreferenceStore store)
	{
		// Tracing.
		if(MinesweeperPlugin.getDefault().isDebugging())
			System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Restoring previous state...");		
		
		// Restore mode.
		Preferences preferences = new Preferences();
		String mode = store.getString("type");
		
		if(mode.equals(Constants.MODE_EXPERT))
		{
			preferences.setMinesCount(Constants.MINES_MODE_EXPERT);
			preferences.setRows(Constants.ROWS_MODE_EXPERT);
			preferences.setColumns(Constants.COLUMNS_MODE_EXPERT);
			preferences.setMode(Constants.MODE_EXPERT);
		}
		else if(mode.equals(Constants.MODE_INTERMEDIATE))
		{
			preferences.setMinesCount(Constants.MINES_MODE_INTERMEDIATE);
			preferences.setRows(Constants.ROWS_MODE_INTERMEDIATE);
			preferences.setColumns(Constants.COLUMNS_MODE_INTERMEDIATE);
			preferences.setMode(Constants.MODE_INTERMEDIATE);
		}
		else if(mode.equals(Constants.MODE_BEGINNER))
		{
			preferences.setMinesCount(Constants.MINES_MODE_BEGINNER);
			preferences.setRows(Constants.ROWS_MODE_BEGINNER);
			preferences.setColumns(Constants.COLUMNS_MODE_BEGINNER);
			preferences.setMode(Constants.MODE_BEGINNER);
		}
		else
		{
			// Put the rest of ............
			preferences.setMinesCount(store.getInt(Constants.MINES_MODE_CUSTOM));
			preferences.setRows(store.getInt(Constants.ROWS_MODE_CUSTOM));
			preferences.setColumns(store.getInt(Constants.COLUMNS_MODE_CUSTOM));
			preferences.setMode(Constants.MODE_CUSTOM);
		}			

		// Restore records.
		Hashtable records = new Hashtable();

		records.put(Constants.RECORD_NAME_MODE_EXPERT,store.getString(Constants.RECORD_NAME_MODE_EXPERT));
		records.put(Constants.RECORD_TIME_MODE_EXPERT,new Integer(store.getInt(Constants.RECORD_TIME_MODE_EXPERT)));			
		records.put(Constants.RECORD_NAME_MODE_INTERMEDIATE,store.getString(Constants.RECORD_NAME_MODE_INTERMEDIATE));
		records.put(Constants.RECORD_TIME_MODE_INTERMEDIATE,new Integer(store.getInt(Constants.RECORD_TIME_MODE_INTERMEDIATE)));			
		records.put(Constants.RECORD_NAME_MODE_BEGINNER,store.getString(Constants.RECORD_NAME_MODE_BEGINNER));
		records.put(Constants.RECORD_TIME_MODE_BEGINNER,new Integer(store.getInt(Constants.RECORD_TIME_MODE_BEGINNER)));						
		
		// Create the controller
		this.controller = new Controller(this,preferences,records);
	}

	// Save state.
	public void saveState(IPreferenceStore store)
	{
		if(MinesweeperPlugin.getDefault().isDebugging())
			System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Saving the state...");
		
		// Save mode.
		if(controller.getPreferences().getMode().equals(Constants.MODE_EXPERT))
			store.setValue("type",Constants.MODE_EXPERT);
		else if(controller.getPreferences().getMode().equals(Constants.MODE_INTERMEDIATE))
			store.setValue("type",Constants.MODE_INTERMEDIATE);
		else if(controller.getPreferences().getMode().equals(Constants.MODE_BEGINNER))
			store.setValue("type",Constants.MODE_BEGINNER);
		else
		{
			store.setValue("type",Constants.MODE_CUSTOM);
			store.setValue(Constants.MINES_MODE_CUSTOM,controller.getPreferences().getMinesCount());
			store.setValue(Constants.ROWS_MODE_CUSTOM,controller.getPreferences().getRows());			
			store.setValue(Constants.COLUMNS_MODE_CUSTOM,controller.getPreferences().getColumns());			
		}

		// Save records.
		Hashtable records = controller.getRecords();

		store.setValue(Constants.RECORD_NAME_MODE_EXPERT,(String)records.get(Constants.RECORD_NAME_MODE_EXPERT));
		store.setValue(Constants.RECORD_TIME_MODE_EXPERT,((Integer)records.get(Constants.RECORD_TIME_MODE_EXPERT)).intValue());		
		store.setValue(Constants.RECORD_NAME_MODE_INTERMEDIATE,(String)records.get(Constants.RECORD_NAME_MODE_INTERMEDIATE));
		store.setValue(Constants.RECORD_TIME_MODE_INTERMEDIATE,((Integer)records.get(Constants.RECORD_TIME_MODE_INTERMEDIATE)).intValue());		
		store.setValue(Constants.RECORD_NAME_MODE_BEGINNER,(String)records.get(Constants.RECORD_NAME_MODE_BEGINNER));
		store.setValue(Constants.RECORD_TIME_MODE_BEGINNER,((Integer)records.get(Constants.RECORD_TIME_MODE_BEGINNER)).intValue());				
	}

	// Free all resources.	
	public void dispose()
	{
		this.saveState(MinesweeperPlugin.getDefault().getPreferenceStore());
		
		// Tracing.
		if(MinesweeperPlugin.getDefault().isDebugging())
			System.out.println("[Minesweeper - " + Constants.TRACE_INFO + "] - " + Calendar.getInstance().getTime() + " - Disposing resources...");
		
		super.dispose();
		ResourceManager.getResourceManager().dispose();
	}
	
	// Getters
	public Button getMainButton()
	{
		return this.mainButton;
	}
	
	public Counter getCounter()
	{
		return this.counter;
	}
	
	public Controller getController()
	{
		return this.controller;
	}
	
	public Timer getTimer()
	{
		return this.timer;
	}
	
	public Composite getMinesField()
	{
		return minesField;
	}
}