/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.minesweeper.ui;

import es.org.chemi.games.util.Constants;
import java.util.Hashtable;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class NewRecordDialog extends Dialog implements ModifyListener
{
	private MainView view = null;
	private Button ok = null;
	private Text name = null;
	
	/**
	 * Constructor for NewRecordDialog
	 */
	public NewRecordDialog(Shell shell, MainView view) 
	{
		super(shell);
		this.view = view;
	}
	
	protected void configureShell(Shell newShell) 
	{
 	   super.configureShell(newShell);
    	newShell.setText("New Best Time");
	}
	
	protected void createButtonsForButtonBar(Composite parent) 
	{
		// create OK and Cancel buttons by default
		ok = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		ok.setEnabled(false);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}
		
	protected Control createDialogArea(Composite parent) 
	{
	    Composite composite = (Composite)super.createDialogArea(parent);
	    
	    GridLayout gridLayout = new GridLayout();
	    gridLayout.numColumns = 1;
	    gridLayout.verticalSpacing = 20;
	    gridLayout.marginWidth = 45;
	    
	    composite.setLayout(gridLayout);
	    
	    Label aux = new Label(composite,SWT.NONE);
	    aux.setAlignment(SWT.CENTER);
	    aux.setText("You have the fastest time\nfor beginner level.");
	    aux = new Label(composite,SWT.CENTER);
	    aux.setText("Please enter your name:");
	    name = new Text(composite,SWT.BORDER);
	    GridData gridData = new GridData();
	    gridData.horizontalAlignment = GridData.CENTER;
	    name.setLayoutData(gridData);
		name.addModifyListener(this);
	    
    	return composite;
	}
	
	public void modifyText(ModifyEvent e)
	{
		if(name.getText().equals(""))
			ok.setEnabled(false);
		else
			ok.setEnabled(true);
	}
	
	protected void okPressed()
	{
		String mode = view.getController().getPreferences().getMode();

		if(mode.equals(Constants.MODE_EXPERT))
		{
			((Hashtable)view.getController().getRecords()).put(Constants.RECORD_NAME_MODE_EXPERT,name.getText());
			((Hashtable)view.getController().getRecords()).put(Constants.RECORD_TIME_MODE_EXPERT,new Integer(view.getTimer().getTime()));			
		}
		else if(mode.equals(Constants.MODE_INTERMEDIATE))
		{
			((Hashtable)view.getController().getRecords()).put(Constants.RECORD_NAME_MODE_INTERMEDIATE,name.getText());
			((Hashtable)view.getController().getRecords()).put(Constants.RECORD_TIME_MODE_INTERMEDIATE,new Integer(view.getTimer().getTime()));			
		}
		else if(mode.equals(Constants.MODE_BEGINNER))
		{
			((Hashtable)view.getController().getRecords()).put(Constants.RECORD_NAME_MODE_BEGINNER,name.getText());
			((Hashtable)view.getController().getRecords()).put(Constants.RECORD_TIME_MODE_BEGINNER,new Integer(view.getTimer().getTime()));			
		}
		super.okPressed();	
		
		RecordsDialog dialog = new RecordsDialog(view.getViewSite().getShell(), view);
		dialog.open();		
	}
}

