/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.minesweeper.util;

import es.org.chemi.games.minesweeper.MinesweeperPlugin;
import es.org.chemi.games.util.Constants;
import es.org.chemi.games.util.ResourceManager;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;

public class Counter extends Composite
{
	private int minesCount = 0;
	private int totalCount = 0;
	private Label hundredsLabel = null;
	private Label tensLabel = null;
	private Label unitsLabel = null;	

	private Composite parent = null;
	private Composite container = null;
	
	public Counter(Composite parent, int style, int value)
	{
		super(parent,style);
		this.parent = parent;
		this.minesCount = value;
		this.totalCount = value;
		
		RowLayout counterLayout = new RowLayout();
		counterLayout.justify = true;
		setLayout(counterLayout);		
		
		// Create a container for counter.
		this.container = new Composite(this,SWT.BORDER);
		RowLayout containerLayout = new RowLayout();
		containerLayout.spacing = 0;
		containerLayout.marginTop = 0;
		containerLayout.marginBottom = 0;
		containerLayout.marginLeft = 0;
		containerLayout.marginRight = 0;
		this.container.setLayout(containerLayout);

		// Loading images.
		URL imageLocation = null;
		URL aux = null;
		
		try
		{
			aux = new URL(MinesweeperPlugin.getDefault().getDescriptor().getInstallURL(),"icons/");
			imageLocation = new URL(aux,"0.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(0),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"1.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(1),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"2.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(2),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"3.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(3),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"4.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(4),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"5.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(5),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"6.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(6),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"7.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(7),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"8.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(8),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"9.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(9),new Image(null,imageLocation.openConnection().getInputStream()));
		}
		catch(MalformedURLException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));
		}
		catch(IOException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));
		}
		
		// Initialice the counter.
		this.hundredsLabel = new Label(container,SWT.NONE);
		this.hundredsLabel.setLayoutData(new RowData(13,23));
		this.tensLabel = new Label(container,SWT.NONE);
		this.tensLabel.setLayoutData(new RowData(13,23));
		this.unitsLabel = new Label(container,SWT.NONE);
		this.unitsLabel.setLayoutData(new RowData(13,23));		
		this.drawCounter(minesCount);
	}
	
	public void drawCounter(int param)
	{
		// Calculate the images to be displayed.	
		int hundreds = param / 100;
		int tens = (param - (hundreds*100)) / 10;
		int units = param - (hundreds*100) - (tens*10);	
					
		// Show images.
		this.hundredsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(hundreds)));
		this.tensLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(tens)));
		this.unitsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(units)));
	}
	
	public void increase()
	{
		this.minesCount += 1;
		this.drawCounter(minesCount);
	}
	
	public void decrease()
	{
		this.minesCount -= 1;
		this.drawCounter(minesCount);
	}
	
	public void reset()
	{
		this.minesCount = this.totalCount;
		this.drawCounter(minesCount);		
	}
	
	public void reset(int value)
	{
		this.totalCount = value;
		this.minesCount = this.totalCount;
		this.drawCounter(minesCount);				
	}
	
	public boolean isZero()
	{
		if(this.minesCount == 0)
			return true;
		else
			return false;
	}
}