/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.minesweeper.util;

import es.org.chemi.games.minesweeper.MinesweeperPlugin;
import es.org.chemi.games.util.Constants;
import es.org.chemi.games.util.ResourceManager;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;

public class Timer extends Composite implements Runnable
{
	private boolean running = false;
	private boolean reset = true;
	private int cron = 0;
	private int hundreds = 0;
	private	int oldHundreds = 0;
	private int tens = 0;
	private int oldTens = 0;	
	private int units = 0;
	private Label hundredsLabel = null;
	private Label tensLabel = null;
	private Label unitsLabel = null;	
		
	private Composite parent = null;
	private Composite container = null;
	
	public Timer(Composite parent, int style)
	{
		super(parent,style);
		this.parent = parent;
		
		RowLayout timerLayout = new RowLayout();
		timerLayout.justify = true;
		setLayout(timerLayout);		
		
		// Create a container for counter.
		container = new Composite(this,SWT.BORDER);
		RowLayout containerLayout = new RowLayout();
		containerLayout.spacing = 0;
		containerLayout.marginTop = 0;
		containerLayout.marginBottom = 0;
		containerLayout.marginLeft = 0;
		containerLayout.marginRight = 0;
		container.setLayout(containerLayout);

		// Loading images.
		URL imageLocation = null;
		URL aux = null;
		
		try
		{
			aux = new URL(MinesweeperPlugin.getDefault().getDescriptor().getInstallURL(),"icons/");
			imageLocation = new URL(aux,"0.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(0),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"1.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(1),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"2.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(2),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"3.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(3),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"4.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(4),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"5.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(5),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"6.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(6),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"7.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(7),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"8.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(8),new Image(null,imageLocation.openConnection().getInputStream()));
			imageLocation = new URL(aux,"9.gif");
			ResourceManager.getResourceManager().putImage(Integer.toString(9),new Image(null,imageLocation.openConnection().getInputStream()));
		}
		catch(MalformedURLException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));
		}
		catch(IOException ex)
		{
			MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));		
		}
		
		hundredsLabel = new Label(container,SWT.NONE);
		hundredsLabel.setLayoutData(new RowData(13,23));		
		hundredsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
		
		tensLabel = new Label(container,SWT.NONE);
		tensLabel.setLayoutData(new RowData(13,23));		
		tensLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
		
		unitsLabel = new Label(container,SWT.NONE);
		unitsLabel.setLayoutData(new RowData(13,23));		
		unitsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
	}
	
	public void run()
	{
		// Set the name of the thread.
		Thread.currentThread().setName("Minesweeper - Timer");
		
		while(running)
		{
			try
			{
				// Count seconds.
				Thread.sleep(1000);
				cron++;
				if(cron == 999)	// Stop timer.
					stop();
				
				// Calculate the images to be displayed.	
				hundreds = cron / 100;
				tens = (cron - (hundreds*100)) / 10;
				units = cron - (hundreds*100) - (tens*10);
				
				// Show images.
				parent.getDisplay().syncExec(new Runnable() 
				{
    				public void run() 
    				{
    					if(hundreds != oldHundreds)
    					{
							hundredsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(hundreds)));
							oldHundreds = hundreds;
    					}
						if(tens != oldTens)
						{
							tensLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(tens)));
							oldTens = tens;
						}
						unitsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(units)));							
    				}
				});
			}
			catch(InterruptedException ex)
			{
				MinesweeperPlugin.getDefault().getLog().log(new Status(IStatus.ERROR,MinesweeperPlugin.getDefault().getDescriptor().getUniqueIdentifier(),IStatus.ERROR,ex.toString(),null));				
			}

			// Check if it needs to reset the timer (see resetRunning method).			
			if(reset)
			{
				parent.getDisplay().syncExec(new Runnable() 
				{
    				public void run() 
    				{
						hundredsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
						tensLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
						unitsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));							
						cron = 0;														
						reset = false;
    				}
				});
			}
		}
	}
	
	public boolean isRunning()
	{
		return running;
	}
	
	// Start the timer.
	public void start()
	{
		running = true;
	}
	
	// Stop the timer
	public void stop()
	{
		running = false;
	}
	
	// Reset the timer if it is running.
	public void resetRunning()
	{
		reset = true;
	}
	
	// Reset the timer if it is stoped.
	public void resetStopped()
	{
		hundredsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
		tensLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));
		unitsLabel.setImage(ResourceManager.getResourceManager().getImage(Integer.toString(0)));	
		cron = 0;									
	}
	
	// Get the time.
	public int getTime()
	{
		return cron;
	}
}