/************************************************************
 *
 * Copyright (c) 2001 Chemi. All rights reserved.
 * 
 * This program and the accompanying materials
 * are made available under the terms of the MIT License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/mit-license.html
 *
 ************************************************************/

package es.org.chemi.games.util;

import java.util.Calendar;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Layout;

public class NewGridLayout extends Layout 
{
	// Defining attributes.
	private int newCompositeWidth = 0;				// Current Composite width.
	private int newCompositeHeight = 0;				// Current Composite height.
	private int oldCompositeWidth = 0;				// Previous Composite width.
	private int oldCompositeHeight = 0;				// Previous Composite height.
	private Control compositeChildren[] = null;		// Array with widgets added to the Composite.
	private String traceUI =  Platform.getDebugOption("es.org.chemi.games.minesweeper/debug/ui");	// Tracing
	
	/**
	 * The constructor.
	 */
	public NewGridLayout()
	{
		super();
	}

	/**
	 * The constructor.
	 * @param oldCompositeWidth	int
	 * @param oldCompositeHeight int
	 */
	public NewGridLayout(int oldCompositeWidth, int oldCompositeHeight)
	{
		this.oldCompositeWidth = oldCompositeWidth;
		this.oldCompositeHeight = oldCompositeHeight;
	}

	/**
	 * @see Layout#layout(Composite, boolean)
	 * @param composite Composite
	 * @param flushCache boolean
	 */
	protected void layout(Composite composite, boolean flushCache) 
	{
		// Tracing.
		if(traceUI != null && traceUI.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " - Calling method layout()...");		

		// Check if the View has been minimized so Layout should not work.
		if(composite.getSize().x <= 1 || composite.getSize().y <= 1)  // I use 1 instead of 0 because Motif way of work.
		{
			// Tracing.
			if(traceUI != null && traceUI.equals("true"))
				System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " - View has been minimized or hidden or closed...");		
			return;
		}
		
		// Getting new information.
		newCompositeWidth = composite.getSize().x;
		newCompositeHeight = composite.getSize().y;
		compositeChildren = composite.getChildren();
		
		// Tracing
		// Tracing.
		if(traceUI != null && traceUI.equals("true"))
		{
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   New composite width is: " + newCompositeWidth);		
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   New composite height is: " + newCompositeHeight);		
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   Old composite width is: " + oldCompositeWidth);		
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   Old composite height is: " + oldCompositeHeight);		
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   " + composite.getBounds());					
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " - Composite contains:");					
			for(int i=0; i<compositeChildren.length; i++)
			{
				System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   Children number: " + i + " is: " + compositeChildren[i]);		
				System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " -   Children number: " + i + " bounds are: " + compositeChildren[i].getBounds());						
			}
		}
		
		// Resizing components.
		if(oldCompositeWidth != 0 && oldCompositeHeight != 0)
		{
			for(int i=0; i<compositeChildren.length; i++)
			{
				// Get current state.
				int oldX = compositeChildren[i].getLocation().x;			
				int oldY = compositeChildren[i].getLocation().y;						
				int oldWidth = compositeChildren[i].getSize().x;
				int oldHeight = compositeChildren[i].getSize().y;
				
				// Resize component.
				compositeChildren[i].setBounds(oldX*newCompositeWidth/oldCompositeWidth,oldY*newCompositeHeight/oldCompositeHeight,oldWidth*newCompositeWidth/oldCompositeWidth,oldHeight*newCompositeHeight/oldCompositeHeight);
			}
		}
		else
		{
			// It's first time that layout() is called and the Layout was instantiated without intial parameters.
			for(int i=0; i<compositeChildren.length; i++)
				compositeChildren[i].setBounds(compositeChildren[i].getBounds());
		}

		// Updating new information.
		oldCompositeWidth = newCompositeWidth;
		oldCompositeHeight = newCompositeHeight;	
	}

	/**
	 * @see Layout#computeSize(Composite, int, int, boolean)
	 * @param composite Composite
	 * @param widthHint int
	 * @param heightHint int
	 * @param flushCache boolean
	 */
	protected Point computeSize(Composite composite, int widthHint, int heightHint, boolean flushCache) 
	{
		// Tracing.
		if(traceUI != null && traceUI.equals("true"))
			System.out.println("[Minesweeper - " + Constants.TRACE_UI + "] - " + Calendar.getInstance().getTime() + " - Calling method computeSize()...");		
			
		return new Point(composite.getSize().x,composite.getSize().y);
	}

}

